/**
 * Component Base Class
 * 
 * Sets all arguments passed in to constructor from ComponentLoader
 *
 * Exposes pub/sub methods for triggering events to other components
 *
 */
'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var Component = (function () {
	_createClass(Component, [{
		key: 'defaultData',

		/**
   * Return optional default values for DOM data attributes
   * 
   * @protected
   */
		value: function defaultData() {
			return {
				// camelCased list of data attribute keys and default values
				// myParam: 'myDefaultValue' <- data-my-param="myOverrideValue"
			};
		}

		/**
   * Constructor for the Component
   *
   * Call `super(...arguments);` in the base class constructor
   *
   * @public
   * @param {Node} context - DOM node that contains the component markup
   * @param {Object} data - Optional data object from ComponentLoader.scan()
   * @param {Object} mediator - instance of ComponentLoader for pub/sub
   */
	}]);

	function Component() {
		_classCallCheck(this, Component);

		this.el = arguments[0];
		if (typeof jQuery !== 'undefined') this.$el = jQuery(this.el);
		this.data = arguments[1];
		this.__mediator = arguments[2];

		this._configureData();
	}

	/**
  * Parses the DOM for all data attributes, converts them to camelCase,
  * and applies defaults before storing them in `this.data`
  * 
  * Order of importance of data is as follows:
  * 1. Data passed to constructor using ComponentLoader.scan({})
  * 2. DOM data attributes
  * 3. defaulData() 
     *
  * I.e:
  *  - defaultData() will always be applied if 1) or 2) does not overide the key
  *  - Any data passed to `scan()` will win over DOM attributes or defaultData with same key
  * 
  * @private
  */

	_createClass(Component, [{
		key: '_configureData',
		value: function _configureData() {
			var DOMData = {};
			[].forEach.call(this.el.attributes, function (attr) {
				if (/^data-/.test(attr.name)) {
					var camelCaseName = attr.name.substr(5).replace(/-(.)/g, function ($0, $1) {
						return $1.toUpperCase();
					});
					DOMData[camelCaseName] = attr.value;
				}
			});
			// extend defaults
			this.data = Object.assign(this.defaultData ? this.defaultData() : {}, DOMData, this.data);
		}

		/**
   * Shorthand for binding multiple functions to `this` in one go
   * @param {...Function} func Variable number of function to bind to this context.
   * @protected
   */
	}, {
		key: 'bind',
		value: function bind() {
			var _iteratorNormalCompletion = true;
			var _didIteratorError = false;
			var _iteratorError = undefined;

			try {
				for (var _iterator = arguments[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
					var func = _step.value;

					var funcName = typeof func === 'function' ? func.name : func;
					this[funcName] = this[funcName].bind(this);
				}
			} catch (err) {
				_didIteratorError = true;
				_iteratorError = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion && _iterator['return']) {
						_iterator['return']();
					}
				} finally {
					if (_didIteratorError) {
						throw _iteratorError;
					}
				}
			}
		}

		/**
   * Publish an event for other components
   * @protected
   * @param {String} topic - Event name
   * @param {Object} data - Optional params to pass with the event
   */
	}, {
		key: 'publish',
		value: function publish() {
			var _mediator;

			(_mediator = this.__mediator).publish.apply(_mediator, arguments);
		}

		/**
   * Subscribe to an event from another component
   * @protected
   * @param {String} topic - Event name
   * @param {Function} callback - Function to bind
   */
	}, {
		key: 'subscribe',
		value: function subscribe(topic, callback) {
			this.__mediator.subscribe(topic, callback, this);
		}

		/**
   * Unsubscribe from an event from another component
   * @protected
   * @param {String} topic - Event name
   * @param {Function} callback - Function to unbind
   */
	}, {
		key: 'unsubscribe',
		value: function unsubscribe(topic, callback) {
			this.__mediator.unsubscribe(topic, callback, this);
		}

		/**
   * Utility method for triggering the ComponentLoader to scan the markup for new components
   * @protected
   * @param {Object} data - Optional data to pass to the constructor of any Component initialized by this scan
   */
	}, {
		key: 'scan',
		value: function scan(data) {
			this.__mediator.scan(data);
		}

		/**
   * Utility method for defering a function call
   * @protected
   * @param {Function} callback - Function to call
   * @param {Number} ms - Optional ms to delay, defaults to 17ms (just over 1 frame at 60fps)
   */
	}, {
		key: 'defer',
		value: function defer(callback) {
			var ms = arguments.length <= 1 || arguments[1] === undefined ? 17 : arguments[1];

			setTimeout(callback, ms);
		}

		/**
   * Called by ComponentLoader when component is no longer found in the markup
   * usually happens as a result of replacing the markup using AJAX
   *	
   * Override in subclass and make sure to clean up event handlers etc
   *
   * @protected
   */
	}, {
		key: 'destroy',
		value: function destroy() {}
	}]);

	return Component;
})();

exports['default'] = Component;
module.exports = exports['default'];